==> sagua_0.1-1.tar.gz: sagua-0.1/include/audio.h <==
#ifndef GUARD_audio_h
#define GUARD_audio_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <map>
#include <string>
#include <vector>

#include <cstdlib>
#include <cmath>

#include <ao/ao.h>

#include <glibmm.h>

#include "effect.h"

class MainManager;

class Audio 
{
    public:
	~Audio();

	static std::auto_ptr<Audio> Create(MainManager& mainManager)
	{
	    if(audio_.get() == NULL)
		audio_ = std::auto_ptr<Audio>(new Audio(mainManager));

	    return audio_;
	}

	void SetDriverFromPreference();

	bool TestDriver(const char* const, const std::map<std::string, std::string>&); 

	bool OpenDevice(const unsigned int&, const unsigned int&, const unsigned int&, const unsigned int&);

	void Close()
	{
	    if(playLoopStart_) {
		playLoopStop_ = true;
		playLoopCondPush_.broadcast();
		playLoopThread_->join();
		playLoopStop_ = false;
		
		playLoopStart_ = false;	
	    }
	    
	    if(CheckOpenDevice() && bufferCount_ != 0) {
		
		ao_play(device_, buffer_, bufferCount_);
		
	    }

	    bufferCount_ = 0;

	    memset(buffer_, 0, 142848);
	    
	    ao_close(device_);
	    device_ = NULL;
	}

	bool CheckOpenDevice()
	{
	    if(device_)
		return true;
	    else
		return false;
	}

	bool IsBigEndian()
	{
	    if(ao_is_big_endian() == 1)
		return true;
	    else
		return false;	
	}

	int Play(float** const, const unsigned int&);
	int Play(const std::vector<double>&, const unsigned int&);

    private:
	Audio(MainManager&);
	Audio(const Audio&);
	Audio& operator=(const Audio&);

	static std::auto_ptr<Audio> audio_;

	struct Dither {
	    double dBuffer_[9];
	    double dTState_;
	    int dPos_;
	};

	// mad decoder output size is 4608, ogg vorbis output size is variable,
	// ffmpeg decoder output size is 4096. So i take 4608 * 31, and will 
	// work find.
	char buffer_[142848];
	unsigned int bufferCount_;

	const double doubleInverseRandMax_;
	double ditherRand_;
	double ditherTemp_;
	double ditherXe_;
	double ditherResult_;

	MainManager& mainManager_;

	ao_device* device_;
	ao_option* option_;
	ao_sample_format format_;
	int driverId_;

	Glib::Thread* playLoopThread_;
	Glib::Mutex playLoopMutex_;
	Glib::Cond playLoopCondPush_;
	Glib::Cond playLoopCondPop_;
	bool playLoopStart_;
	bool playLoopStop_;

	unsigned int singleSampleSize_;
	unsigned int bufferSize_;
	bool littleEndian_;

	short int tempScaledSample_;

	Effect::Stereo stereoSignal_;

	Dither dLeft_;
	Dither dRight_;

	void PlayBufferLoop();
	void PlayBufferPush(const char* buf)
	{
	    Glib::Mutex::Lock lock(playLoopMutex_);

	    memcpy(buffer_ + bufferCount_, buf, bufferSize_);

	    bufferCount_ += bufferSize_;
	    
	    // If buffer filled over 4608 * 30, need to flush
	    while(bufferCount_ > 138240 && !playLoopStop_) 
		playLoopCondPop_.wait(playLoopMutex_);

	    playLoopCondPush_.signal();	

	}

	// it is poor and not safe! but for just simple modulo.
	int Mod(const int& x, const int& n)
	{
	    if(x > n)
		return x - n;
	    else
		return x;	
	}

	void ScaleTo16Bit(const double&, Dither&);
	
	void ShortIntToChar(const short int& in, char* out)
	{
	    // becareful address boundry! 
	    if(littleEndian_){
		*out = in & 0xff;
		*(out+1) = in >> 8; 
	    } else {
		*out = in >> 8;
		*(out+1) = in & 0xff;
	    }
	}

	void Reset();

};

#endif

==> sagua_0.1.1-1.tar.gz: sagua-0.1.1/include/audio.h <==
#ifndef GUARD_audio_h
#define GUARD_audio_h
// Sagua is music player.
// Copyright (C) 2004 HG, Rhee <ez8tank@krpost.net>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <map>
#include <string>
#include <vector>

#include <cstdlib>
#include <cmath>

#include <ao/ao.h>

#include <glibmm.h>

#include "effect.h"

class MainManager;

class Audio 
{
    public:
	~Audio();

	static std::auto_ptr<Audio> Create(MainManager& mainManager)
	{
	    if(audio_.get() == NULL)
		audio_ = std::auto_ptr<Audio>(new Audio(mainManager));

	    return audio_;
	}

	void SetDriverFromPreference();

	bool TestDriver(const char* const); 
	bool TestDriver(const char* const, const std::map<std::string, std::string>&); 

	bool OpenDevice(const unsigned int&, const unsigned int&, const unsigned int&, const unsigned int&);

	void Close()
	{
	    if(playLoopStart_) {
		playLoopStop_ = true;
		playLoopCondPush_.broadcast();
		playLoopThread_->join();
		playLoopStop_ = false;
		
		playLoopStart_ = false;	
	    }
	    
	    if(CheckOpenDevice() && bufferCount_ != 0 && !skipBufferFlush_) {
		
		ao_play(device_, buffer_, bufferCount_);
		
	    }

	    bufferCount_ = 0;
	    playLoopPause_ = false;
	    skipBufferFlush_ = false;

	    memset(buffer_, 0, 142848);
	    
	    ao_close(device_);
	    device_ = NULL;
	}

	bool CheckOpenDevice()
	{
	    if(device_)
		return true;
	    else
		return false;
	}

	bool IsBigEndian()
	{
	    if(ao_is_big_endian() == 1)
		return true;
	    else
		return false;	
	}

	void Play(float** const, const unsigned int&);
	void Play(const std::vector<double>&, const unsigned int&);

	void PauseToggle(const bool& skipPlayBufferedSample)
	{
	    if(skipPlayBufferedSample)
		skipBufferFlush_ = true;
	    
	    if(playLoopPause_)
		playLoopPause_ = false;
	    else if(playLoopStart_)
		playLoopPause_ = true;	
	}

	const bool& Paused()
	{
	    return playLoopPause_;
	}

    private:
	Audio(MainManager&);
	Audio(const Audio&);
	Audio& operator=(const Audio&);

	static std::auto_ptr<Audio> audio_;

	struct Dither {
	    double dBuffer_[9];
	    double dTState_;
	    unsigned int dPos_;
	};

	// mad decoder output size is 4608, ogg vorbis output size is variable,
	// ffmpeg decoder output size is 4096. So i take 4608 * 31, and will 
	// work find.
	char buffer_[142848];
	unsigned int bufferCount_;

	const double doubleInverseRandMax_;
	double ditherRand_;
	double ditherTemp_;
	double ditherXe_;
	double ditherResult_;

	MainManager& mainManager_;

	ao_device* device_;
	ao_option* option_;
	ao_sample_format format_;
	int driverId_;

	Glib::Thread* playLoopThread_;
	Glib::Mutex playLoopMutex_;
	Glib::Cond playLoopCondPush_;
	Glib::Cond playLoopCondPop_;
	bool playLoopStart_;
	bool playLoopStop_;
	bool playLoopPause_;
	bool skipBufferFlush_;

	unsigned int singleSampleSize_;
	unsigned int bufferSize_;
	bool littleEndian_;

	short int tempScaledSample_;

	Effect::Stereo stereoSignal_;

	Dither dLeft_;
	Dither dRight_;

	void PlayBufferLoop();
	void PlayBufferPush(const char* buf)
	{
	    Glib::Mutex::Lock lock(playLoopMutex_);

	    memcpy(buffer_ + bufferCount_, buf, bufferSize_);

	    bufferCount_ += bufferSize_;
	    
	    // If buffer filled over 4608 * 30, need to flush
	    while(bufferCount_ > 138240 && !playLoopStop_) 
		playLoopCondPop_.wait(playLoopMutex_);

	    playLoopCondPush_.signal();	

	}

	// it is poor and not safe! but for just simple modulo.
	unsigned int Mod(const unsigned int& x, const unsigned int& n)
	{
	    if(x > n)
		return x - n;
	    else
		return x;	
	}

	void ScaleTo16Bit(const double&, Dither&);
	
	void ShortIntToChar(const short int& in, char* out)
	{
	    // becareful address boundry! 
	    if(littleEndian_){
		*out = in & 0xff;
		*(out+1) = in >> 8; 
	    } else {
		*out = in >> 8;
		*(out+1) = in & 0xff;
	    }
	}

	void Reset();

	class DeviceManager
	{
	    public:
		DeviceManager(): device_(NULL) {}
		DeviceManager(const int& id, ao_sample_format* const format, ao_option* const option)
		: device_(NULL)
		{
		    device_ = ao_open_live(id, format, option);
		}
		~DeviceManager()
		{
		    ao_close(device_);
		}

		const bool OpenLive(const int& id, ao_sample_format* const format, ao_option* const option)
		{
		    device_ = ao_open_live(id, format, option);

		    return device_ != NULL;
		}

		const bool Valid() const
		{
		    return device_ != NULL;
		}

	    private:
		DeviceManager& operator=(const DeviceManager&);
		DeviceManager(const DeviceManager&);
		
		ao_device* device_;	

	};

	class OptionManager
	{
	    public:
		OptionManager(): option_(NULL) {}
		~OptionManager()
		{
		    ao_free_options(option_);
		}

		void Append(const std::string& key, const std::string& value)
		{
		    ao_append_option(&option_, key.c_str(), value.c_str());
		}

		void Append(char* const key, char* const value)
		{
		    ao_append_option(&option_, key, value);
		}

		ao_option* Get()
		{
		    return option_;
		}

	    private:
		OptionManager& operator=(const OptionManager&);
		OptionManager(const OptionManager&);

		ao_option* option_;
	};

	struct AoOptionAppend: std::binary_function<std::pair<std::string, std::string>, OptionManager*, void>
	{
	    void operator()(const std::pair<std::string, std::string>& pair, OptionManager* const option) const
	    {
		option->Append(pair.first, pair.second);
	    }
	};
	
};

#endif
